CD-Title: ROTATION PREDICTION FUNCTIONS (RPF)
CD-end

CD-Intro:
C The rotation image prediction functions are a set of program independent
C Fortran subroutines to enable the prediction of monochromatic X-ray
C crystallographic rotation images. Routines included for generating a
C set of unique reflections for a given cell and space group. The coding
C of the main rotation prediction function is closely based on code from 
C OSCGEN & MADNES by Peter Brick, Andrew Leslie, Jean Claude Thierry, 
C Alan J. Wonacott and J. W. Pflugrath from Reeke prediction coded by 
C R.M. Sweet.
CD-end

CD-Section: Rotation Prediction Functions
C These routines include the main prediction function and some additional
C functions to assist the user with preparing a program/detector specific
C output routine.
CD-end

C LAST UPDATE: 
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C**************************
C**     RPF_PREDICT      **
C**************************
C
CD-Routine: Rotation prediction function - RPF_PREDICT
C The routine rpf_predict is used to reflections for monochromatic rotation 
C method. It is closely Based on code from OSCGEN & MADNES by Peter Brick, 
C Andrew Leslie, Jean Claude Thierry, Alan J. Wonacott and J. W. Pflugrathoth 
C from Reeke prediction coded by R.M. Sweet. In this packaging, the program
C dependent aspsects have been removed. In particular this affects the way
C in which the predicted reflection data are stored once they have been
C predicted and how they relate to spot positions on the detector. The
C user needs to supply a routine to perform this function but some additional
C routines are supplied to assist this process. The prediction routine handles
C systematic absences as determined by the lattice type but the user supplied
C output routine should handle any additional absences.
CD-end
CD-Fortran:
      SUBROUTINE RPF_PREDICT (ROTS, ROTE, APRS, APRE, APRSX, APREX,
     *                 IAX, S0, SCAX, DST2MX, DSTPL2, NWMAX, LATT, 
     *                 ISYN, DIVH, DIVV, ETA, DELAMB, DELCOR,
     *                 IAX_H, IAX_V, QUICK, SAVSPOT, IERR)
CD-end
C
C
C PURPOSE: Predict reflections for monochromatic rotation method
C
C AUTHOR:  Based on code from OSCGEN & MADNES by Peter Brick, Andrew Leslie, 
C          Jean Claude Thierry, Alan J. Wonacott and J. W. Pflugrath from 
C          Reeke prediction coded by R.M. Sweet.
C
C          This version prepared by John W. Campbell, May 1995

C
C ARGUMENTS:
C
      REAL ROTS
      REAL ROTE
      REAL APRS(3,3)
      REAL APRE(3,3)
      REAL APRSX(3,3)
      REAL APREX(3,3)
      INTEGER IAX(3)
      REAL S0(3)
      REAL SCAX(3)
      REAL DST2MX
      REAL DSTPL2
      INTEGER NWMAX 
      INTEGER LATT
      INTEGER ISYN
      REAL DIVH
      REAL DIVV
      REAL ETA
      REAL DELAMB
      REAL DELCOR
      INTEGER IAX_H
      INTEGER IAX_V
      LOGICAL QUICK
      EXTERNAL SAVSPOT
      INTEGER IERR
CD-Parameters:    
C  ROTS       (R)  Rotation start angle in degrees.
C  ROTE       (R)  Rotation end angle in degrees.
C  APRS(3,3)  (R)  Orientation matrix at start of rotation range.
C  APRE(3,3)  (R)  Orientation matrix at end of rotation range.
C  APRSX(3,3) (R)  Orientation matrix at start of range extended (in negative 
C                  direction) by maximum possible reflection width.
C  APREX(3,3) (R)  Orientation matrix at end of range extended (in positive
C                  direction) by maximum possible reflection width.
C  IAX(3)     (R)  Order of varying indices for Reeke algorithm.
C                  IAX(1) r.l. axis most nearly parallel/antiparallel
C                         to X-ray beam.
C                  IAX(3) r.l. axis most nearly parallel/antiparallel to 
C                         rotation axis away from spindle
C                  IAX(2) remaining r.l. axis
C  S0(3)      (R)  Beam direction vector wrt laboratory axes.
C  SCAX(3)    (R)  Scan axis direction vector  wrt laboratory axes.
C  DST2MX     (R)  Maximum value of (dstar)**2
C  DSTPL2     (R)  Maximum value of (dstar)**2 allowing for a bit extra for
C                  determining overlaps at edge of image
C  NWMAX      (R)  Maximum no. of images over which a spot may be spread.
C  LATT       (R)  Lattice type flag 1-7 for P, A, B, C, I, F, R for lattice
C                  based systematic absences. User must handle other space
C                  group specific absences in user supplied SAVSPOT routine.
C  ISYN       (R)  Source type flag =0 lab source, =1 synchrotron.
C  DIVH       (R)  Horizontal beam divergence (half-width, radians)
C  DIVV       (R)  Verticaal beam divergence (half-width, radians)
C  ETA        (R)  Mosaicity (half_width, radians).
C  DELAMB     (R)  Dispersion (Delta(lambda)/lambda)
C  DELCOR     (R)  Correlated dispersion term (Only used for synchrotron)
C                  (half-width, radians)
C  IAX_H      (R)  No. of horizontal axis in laborarory system (1=X, 2=Y, 3=Z)
C  IAX_V      (R)  No. of vertical axis in laborarory system (1=X, 2=Y, 3=Z)
C  QUICK      (R)  Logical flag; If .true. do a quick analysis which does
C                  not distinguish between fulls & partials and which does
C                  not calculate relection widths. If .false. do the full
C                  detailed prediction.
C  SAVSPOT    (S)  Routine to convert spot coordinates to detector coordinates
C                  & save  spot details as desired. This user supplied routine
C                  may call RPF_RTOD if appropriate. Also, if a detailed
C                  prediction has been done (QUICK = .false.) RPF_FRACS 
C                  may be called to obtain additional items of information 
C                  about the predicted reflection not passed via the 
C                  subroutine parameters. The routine should also check
C                  for any systematic absences in addition to those due to
C                  the basic lattice type. The parameters in the call depend
C                  on the value of the flag QUICK and are as follows:
C
C                  if QUICK = .FALSE. (detailed analysis requested)
C
C                  CALL SAVSPOT (IHKL, IR_FLAG, INR_FLAG, NIMG, IFIRST, 
C                 +              XRLPE, PHI, PHIW, RLORF, DST2MX, IERR)
C
C                  INTEGER IHKL(3)    (R)  Reflection indices
C                  INTEGER IR_FLAG    (R)  Recorded flag      
C                                              0 = not recorded
C                                              1 = full
C                                              2 = good partial 
C                                                  (<= nwmax images)
C                                              3 = too-wide partial
C                                              4 = within cusp but part 
C                                                  visible
C                  INTEGER INR_FLAG   (R)  Not recorded flag   
C                                              0 = recorded
C                                              1 = on rotation axis
C                                              2 = within cusp
C                                              3 = > extended resol limit
C                                              4 = not-recorded but still 
C                                                  in list
C                  INTEGER NIMG       (R)  No. of images over which spot 
C                                          spread (1=full spot)
C                                          0 if too wide (>NWMAX images)
C                  INTEGER IFIRST     (R)  First image for spot (offset from 
C                                          current image)
C                  REAL XRLPE(3)      (R)  Reciprocal lattice coordinates for
C                                          reflection on Ewald sphere wrt the 
C                                          laboratory axes.
C                  REAL PHI           (R)  Phi value when rlp is on the Ewald
C                                          sphere
C                  REAL PHIW          (R)  Reflection width
C                  REAL RLORF         (R)  Lorentz factor
C                  DST2MX             (R)  Maximum value of (dstar)**2 as
C                                          used in RPF_PREDICT call
C                  IERR               (W)  Error flag (returned via
C                                          RPF_PREDICT parameter IERR)
C                                          Must be set to 0 if OK and
C                                          a non-zero value for an error.
C
C                  if QUICK = .TRUE. (quick analysis requested)
C
C                  CALL SAVSPOT (IHKL, IR_FLAG, INR_FLAG, PHI, DST2MX,
C                 +              IERR)
C
C                  INTEGER IHKL(3)    (R)  Reflection indices
C                  INTEGER IR_FLAG    (R)  Recorded flag      
C                                              0 = not recorded
C                                              1 = recorded (full/partial)
C                  INTEGER INR_FLAG   (R)  Not recorded flag   
C                                              0 = recorded
C                                              1 = on rotation axis
C                                              2 = within cusp
C                  REAL PHI           (R)  Phi value when rlp is on the Ewald
C                                          sphere
C                  DST2MX             (R)  Maximum value of (dstar)**2 as
C                                          used in RPF_PREDICT call
C                  IERR               (W)  Error flag (returned via
C                                          RPF_PREDICT parameter IERR)
C                                          Must be set to 0 if OK and
C                                          a non-zero value for an error.
C
C  IERR       (W)  Error flag from SAVSPOT routine
CD-end
C
C====== Parameter statements
C 
      REAL       RD2DG, DG2RD
      PARAMETER (RD2DG = 180. / 3.14159265 )
      PARAMETER (DG2RD = 3.14159265 / 180.0)
C
C====== GLOBALS:
C

C
C====== FUNCTION TYPES:
C

      REAL       DOT
C
C====== LOCALS:
C
      LOGICAL    EXTNCT
      LOGICAL    ABSNT
      INTEGER    IRLB(2), IRLE(2)
      INTEGER    NRTA(2)
      INTEGER    IHKL(3), IH, IK, IL, IER
      INTEGER    INTB, INTE
      INTEGER    IR_FLAG, INR_FLAG, NIMG, IFIRST
      INTEGER    IPLB, IPLE, IP
      INTEGER    IQMIN, IQMAX, IQLE, IQLB, IQ
      INTEGER    IRMIN, IRMAX, IR
      INTEGER    I, J, K, L, J1, J2, ISIGN
      INTEGER    NRT, NLOOP, ILOOP
      INTEGER    ISYS, KSYS(3)
      INTEGER    KERR
      REAL       PMN(2), PMX(2), AVECX, VECX
      REAL       P0(3,3,2), P(3,3,2)
      REAL       PP(3,4,2), T(4,4), VEC(3),
     *           RT(2), RS1(2), RS0(2), RU2(2), RU1(2), RU0(2), QU(2),
     *           P1H(2), P2H(2), P3H(2), RLB(2), RLE(2),
     *           P1H0(2), P2H0(2), P3H0(2), PX0(2), PY0(2), PZ0(2),
     *           X12(2), Y12(2), Z12(2)
      REAL       STARTR, ENDR, XR(3)
      REAL       PHI, PHIW, PHIRNG, POL, FLOP, DSTSQ
      REAL       XRLPE(3), SCAX_UN(3)
      REAL       A, A1, SCA1
      REAL       XDET, YDET
      REAL       P1, P2, PX1, PX2, T1, T2, AVEC, SIN2T, SINSQ, SINTH
      REAL       S1, S2, S3, S4, S5, S6, S1P2R2, S5P, S6P
      REAL       PA, PB, PC, PD, PMIN, PMAX, PMINE, PMAXE
      REAL       FP, FP2, PX, PY, PZ
      REAL       QMINE, QMAXE, FQ
      REAL       FR
      REAL       RLORF


      EQUIVALENCE (IHKL(1),IH), (IHKL(2),IK), (IHKL(3),IL)
      DATA         PP /24*0.0/
C
C-------------------------------------------------------------------------------
C
C
C  Statement functions
C
      INTB(A) = INT(A - 1.0)
      INTE(A) = INT(A + 1.0)
C
C  Set flags for systematic absences
C
      KSYS(1) = 0
      KSYS(2) = 0
      KSYS(3) = 0
      IF (LATT.EQ.1) THEN
         ISYS = 0
      ELSE IF (LATT.EQ.2) THEN
         ISYS = 2
         KSYS(2) = 1
         KSYS(3) = 1
      ELSE IF (LATT.EQ.3) THEN
         ISYS = 2
         KSYS(1) = 1
         KSYS(3) = 1
      ELSE IF (LATT.EQ.4) THEN
         ISYS = 2
         KSYS(1) = 1
         KSYS(2) = 1
      ELSE IF (LATT.EQ.5) THEN
         ISYS = 2
         KSYS(1) = 1
         KSYS(2) = 1
         KSYS(3) = 1
      ELSE IF (LATT.EQ.6) THEN
         ISYS = 4
      ELSE IF (LATT.EQ.7) THEN
         ISYS = 3
         KSYS(1) = -1
         KSYS(2) = 1
         KSYS(3) = 1
      END IF
C
C   Clear error flag
C
      IERR = 0
C
C   Get SCAX as a unit vector
C
      SCAX_UN(1) = SCAX(1)
      SCAX_UN(2) = SCAX(2)
      SCAX_UN(3) = SCAX(3)
      CALL UNIT(SCAX_UN)
C
C  Compute start and end of rotation range in radians:
C
      STARTR = ROTS * DG2RD
      ENDR   = ROTE * DG2RD
C
C
C  Consider axis IAX(1) to be IP, IAX(2) to be IQ, IAX(3) to be IR
C
C  Fix up ordering of setting matrices to be in column
C  order IP, IQ, IR
C
      DO 2 K = 1, 3
         DO 2 J = 1, 3
c...jwc... next line changed
            P(J,K,1) = APRSX(J,IAX(K))
            P0(J,K,1) = APRS(J,IAX(K))
            P(J,K,2) = APREX(J,IAX(K))
            P0(J,K,2) = APRE(J,IAX(K))
 2       CONTINUE
      CONTINUE
C
C  Set PP (Pprime) as a 3x4(x2) matrix. Translation to move origin
C  to center of the Ewald sphere.  S0 is a unit vector antiparallel
C  to the X-ray beam.
C
      DO 30 I = 1, 2
         DO 20 K = 1, 3
            PP(K,4,I) = -S0(K)
            DO 10 J = 1, 3
               PP(K,J,I) = P(K,J,I)
 10         CONTINUE
 20      CONTINUE
 30   CONTINUE
C
C  Calculate the reciprocal metric tensor T = PP(trans) . PP
C  The 3*3 portion of matrix is same at beginning and end of rotation
C  Note that RT, RS1, RU2 and QU don't depend on L and thus don't need
C  to be calculated twice.
C  PP is used here to calculate T but is not used again
C
      DO 80 L = 1, 2
         DO 70 I = 1, 4
            DO 70 J = 1, 4
               T1 = 0.0
               DO 60 K = 1, 3
 60               T1 = T1  +  PP(K,I,L) * PP(K,J,L)
               CONTINUE
 70            T(I,J) = T1
            CONTINUE
         CONTINUE
C
         RT(L)  =        T(2,3)**2       -  T(3,3) * T(2,2)
         RS1(L) =        T(1,3) * T(2,3) -  T(3,3) * T(1,2)
         RS0(L) =        T(2,3) * T(3,4) -  T(2,4) * T(3,3)
         RU2(L) =        T(1,3)**2       -  T(1,1) * T(3,3)
         RU1(L) = 2.0 * (T(1,3) * T(3,4) -  T(1,4) * T(3,3))
         RU0(L) = T(3,4)**2
         QU(L)  = T(3,3)
 80   CONTINUE
C
      S1 = T(1,1)
      S2 = T(2,2)
      S3 = T(3,3)
      S4 = T(3,2) + T(2,3)
      S5 = T(1,3) + T(3,1)
      S6 = T(1,2) + T(2,1)
C
      PA = S4 * S4  -  4.0 * S2 * S3
      PB = S4 * S5  -  2.0 * S3 * S6
      PC = S5 * S5  -  4.0 * S1 * S3
      PD = 4.0 * S3 * DSTPL2

C
C  Initialisation complete
C
C  Get IP looping limits
C
      PMIN = 1.E20
      PMAX = -1.E20
      SINTH = 0.5 * SQRT(DSTPL2)
      SINSQ = SINTH**2
      SIN2T = SIN(2.0 * ATAN2(SINTH, SQRT(1.0-SINSQ)))
C
C  Ewald sphere limit and limiting sphere both computed here.
C  Note that in order to allow for a general X-ray beam direction,
C  the algorithm for calculating the limiting sphere index limits
C  has been changed from that proposed by Reeke, and is now
C  essentially the same as the algorithm used for the Ewald sphere.
C
      PMINE =  1.E20
      PMAXE = -1.E20
C
      DO 120 I = 1, 2
C
C  Determine if the reciprocal latttice vector closest to the X-ray
C  beam is parallel or antiparallel. Remember S0 is antiparallel to
C  X-ray beam !
C
         SCA1 =  DOT(P(1,1,I), S0)
         ISIGN = -INT( SIGN(1.1, SCA1) )
         DO 100 J = 1, 3
            J1 = MOD(J,3) + 1
            J2 = MOD(J+1,3) + 1
 100        VEC(J) = P(J1,2,I) * P(J2,3,I)  -  P(J1,3,I) * P(J2,2,I)
         CONTINUE
C
C  VEC is the vector normal to the reciprocal lattice planes with
C  IP index zero.
C
         T1 = 0.0
         T2 = 0.0
         DO 110 J = 1,3
            T1 = T1  +  VEC(J)**2
 110        T2 = T2  +  P(J,1,I) * VEC(J)
         CONTINUE
C
C  T2 is the volume of the reciprocal space unit cell. Because of
C  the permutation of axes (by IAX), this may be negative.
C
         AVEC = SQRT(T1)
         T2 = 1.0 / ABS(T2)
C
C  Limiting sphere limits
C
         VECX =  DOT(S0, VEC)
         AVECX = ABS(VECX)
         A1 = T1  -  AVECX * AVECX
         IF (A1 .GT. 0.0) THEN
            A1 = SQRT(A1)
         ELSE
            A1 = 0.0
         ENDIF
         PX1 = -ISIGN * T2 * (2. * SINSQ * AVECX  +  SIN2T * A1)
         PX2 = -ISIGN * T2 * (2. * SINSQ * AVECX  -  SIN2T * A1)
         P1 = AMIN1(PX1, PX2)
         P2 = AMAX1(PX1, PX2)
C
C  Ewald sphere limits
C
         T1 = -ISIGN * T2 * (AVEC + AVECX)
         T2 =  ISIGN * T2 * (AVEC - AVECX)
         PMINE = AMIN1(T1, T2)
         PMAXE = AMAX1(T1, T2)
C
C  Select limits on basis of ISIGN
C
         IF (ISIGN .GT. 0) THEN
            PMN(I) = AMAX1(P1, PMINE)
            PMX(I) = AMAX1(P2, PMAXE)
         ELSE
            PMN(I) = AMIN1(P1, PMINE)
            PMX(I) = AMIN1(P2, PMAXE)
         ENDIF
 120  CONTINUE
C
C  Select overall limits
C
      PMIN = AMIN1(PMN(1), PMN(2))
      PMAX = AMAX1(PMX(1), PMX(2))
      IPLB = INTB(PMIN)
      IPLE = INTE(PMAX)
C
C  Loop over IP
C
      DO 460 IP = IPLB, IPLE
C
C  Calculate a few P-dependent constants
C
         FP = FLOAT(IP)
         FP2 = FP * FP
         S5P = S5 * FP
         S6P = S6 * FP
         S1P2R2 = FP2 * S1  -  DSTPL2
C
C  Calculate looping limits for IQ
C            limiting sphere
C
         CALL RPF_QD2(PA, FP * PB, (FP2 * PC  +  PD), VEC, NRT)
C
         IF (NRT - 1) 460, 150, 160
 150        IQMIN = INTB(VEC(1))
            IQMAX = INTE(VEC(1))
         GOTO 170
C
 160        IQMIN = INTB( AMIN1(VEC(1), VEC(2)) )
            IQMAX = INTE( AMAX1(VEC(1), VEC(2)) )
C
C  Ewald sphere
C
 170     QMINE = 1.E20
         QMAXE = -1.E20
C
c---jwc---         DO 195 I = 1, 3
c---jwc--- 195        P1H0(I) = P0(I,1,1) * FP
c---jwc---         CONTINUE
         DO 200 I = 1, 2
            P1H0(I) = P0(1,1,I)*FP
            P2H0(I) = P0(2,1,I)*FP
            P3H0(I) = P0(3,1,I)*FP
            P1H(I) = P(1,1,I) * FP  -  S0(1)
            P2H(I) = P(2,1,I) * FP  -  S0(2)
            P3H(I) = P(3,1,I) * FP  -  S0(3)
            T1 =     RU0(I)  +  FP * (RU1(I)  +  FP * RU2(I))
C
            CALL RPF_QD2(RT(I), (FP*RS1(I) + RS0(I)), T1, VEC, NRTA(I))
C
            IF (NRTA(I) - 1) 200, 180, 190
 180           QMINE = AMIN1(QMINE, VEC(1))
               QMAXE = AMAX1(QMAXE, VEC(1))
            GOTO 200
C
 190           QMINE = AMIN1(QMINE, VEC(1), VEC(2))
               QMAXE = AMAX1(QMAXE, VEC(1), VEC(2))
 200        CONTINUE
C
            IF (NRTA(1) .EQ. 0  .AND.  NRTA(2) .EQ. 0) GOTO 460
C
C  Choose most restrictive limits
C
            IQLB = INTB(QMINE)
            IQLE = INTE(QMAXE)
            CALL RPF_LIM(IQMIN, IQMAX, IQLB, IQLE, IER)
            IF (IER .NE. 0) GOTO 460
C
C  Loop over IQ
C
            DO 450 IQ = IQLB, IQLE
               FQ = FLOAT(IQ)
C
C  Get looping limits for IR
C            D* limit
C
               T1 = S1P2R2  +  FQ * (FQ * S2  +  S6P)
C
               CALL RPF_QD2(S3, (0.5 * (S4 * FQ + S5P)), T1, VEC, NRT)
C
               IF (NRT - 1) 450, 210, 220
 210              IRMIN = INTB(VEC(1))
                  IRMAX = INTE(VEC(1))
               GOTO 230
C
 220              IRMIN = INTB(AMIN1(VEC(1), VEC(2)))
                  IRMAX = INTE(AMAX1(VEC(1), VEC(2)))
C
C  Ewald sphere limits
C
 230           DO 260 I = 1, 2
                  PX = P1H(I)  +  P(1,2,I) * FQ
                  PY = P2H(I)  +  P(2,2,I) * FQ
                  PZ = P3H(I)  +  P(3,2,I) * FQ
                  T1 = PX * PX  +  PY * PY  +  PZ * PZ  -  1.0
                  T2 = P(1,3,I) * PX  +  P(2,3,I) * PY  +  P(3,3,I) * PZ
C
                  CALL RPF_QD2(QU(I), T2, T1, VEC, NRTA(I))
C
                  IF (NRTA(I) - 1) 260, 240, 250
 240                 RLB(I) = VEC(1)
                     RLE(I) = VEC(1)
                  GOTO 260
C
 250                  RLB(I) = AMIN1(VEC(1), VEC(2))
                      RLE(I) = AMAX1(VEC(1), VEC(2))
 260              CONTINUE
C
C  Use quadratic roots to set up loop limits depending on sphere
C  intersections and adjust loops to other limits.
C
                  NLOOP = 1
                  IF (NRTA(1) .GT. 0) GOTO 270
                     IF (NRTA(2) .LE. 0) GOTO 450
                        NRTA(1) = NRTA(2)
                        RLB(1) = RLB(2)
                        RLE(1) = RLE(2)
                     GOTO 300
C
 270              IF (NRTA(2) .LE. 0) GOTO 300
                     IF (RLB(1) .LE. RLB(2)) GOTO 280
                        CALL RPF_ISW(NRTA(1), NRTA(2))
                        CALL RPF_RSW(RLB(1), RLB(2))
                        CALL RPF_RSW(RLE(1), RLE(2))
 280                 IF (NRTA(1) .EQ. 1) GOTO 290
                        IF (NRTA(2) .EQ. 2) GOTO 310
 290                 NLOOP = 2
 300              CALL RPF_LPS(NLOOP, RLB(1), RLE(1), RLB(2),
     *                               RLE(2), IRLB, IRLE)
                  GOTO 330
C
 310              IF (RLE(2) .GT. RLE(1)) GOTO 320
                     NLOOP = 2
                     CALL RPF_LPS(NLOOP, RLB(1), RLB(2), RLE(2),
     *                                  RLE(1), IRLB, IRLE)
                  GOTO 330
C
 320              IF (RLB(2) .GE. RLE(1)) GOTO 290
                     NLOOP = 2
                     CALL RPF_LPS(NLOOP, RLB(1), RLB(2), RLE(1),
     *                                  RLE(2), IRLB, IRLE)
C
C  Check IR limits
C
 330              CALL RPF_LIM(IRMIN, IRMAX, IRLB(1), IRLE(1), IER)
                  IF (IER .NE. 0) GOTO 350
                  IF (NLOOP .EQ. 1) GOTO 370
 340                 CALL RPF_ISW(IRLB(1), IRLB(2))
                     CALL RPF_ISW(IRLE(1), IRLE(2))
                     CALL RPF_LIM(IRMIN, IRMAX, IRLB(1), IRLE(1), IER)
                     IF (IER .EQ. 0) GOTO 370
                  GOTO 360
C
 350                 IF (NLOOP .EQ. 1) GOTO 450
                     NLOOP = 1
                  GOTO 340
C
 360              IF (NLOOP .EQ. 1) GOTO 450
                  CALL RPF_ISW(IRLB(1), IRLB(2))
                  CALL RPF_ISW(IRLE(1), IRLE(2))
                  NLOOP = 1
C
C  Loop over IR
C
 370              CONTINUE
C
c---jwc---                  DO 380 I = 1, 3
c---jwc--- 380                 PX0(I) = P1H0(I)  +  P0(I,2,1) * FQ
c---jwc---                  CONTINUE
                  DO 380 I = 1,2
                     PX0(I) = P0(1,2,I)*FQ + P1H0(I)
                     PY0(I) = P0(2,2,I)*FQ + P2H0(I)
                     PZ0(I) = P0(3,2,I)*FQ + P3H0(I)
 380             CONTINUE
C
                  DO 440 ILOOP = 1, NLOOP
                     DO 440 IR = IRLB(ILOOP), IRLE(ILOOP)
                        IHKL(IAX(1)) = IP
                        IHKL(IAX(2)) = IQ
                        IHKL(IAX(3)) = IR
C
C  Test for any systematic absences (...jwc... from centering)
C
c...jwc...
                        ABSNT = .FALSE.
                        IF (ISYS.GT.0) THEN

                           IF (ISYS.EQ.4) THEN
                              ABSNT = .TRUE.
                              DO 382 I = 1,2
                                 IF (MOD(IHKL(I)+IHKL(I+1),2).NE.0) 
     +                               GO TO 385
 382                          CONTINUE
                              ABSNT = .FALSE.
                           ELSE
C
C---- This version for A,B,C,R,I & F lattice absences
C
                              ABSNT = .FALSE.
                              J = 0
                              DO 384 I = 1,3
                                 J = IHKL(I)*KSYS(I) + J
 384                          CONTINUE
                              IF (MOD(J,ISYS).NE.0) ABSNT = .TRUE.
                           END IF
                        END IF
 385                    IF (ABSNT) GO TO 440
C
                        FR = FLOAT(IR)
C
C  X12 etc contains the coords of the rlp at the start and end of the rotation
C  range
C
                        DO 390 I = 1, 2
                        X12(I) = PX0(I)  +  P0(1,3,I) * FR
                        Y12(I) = PY0(I)  +  P0(2,3,I) * FR
                        Z12(I) = PZ0(I)  +  P0(3,3,I) * FR
 390                    CONTINUE
C
C  Test for valid reflexion within D* limit
C  calculate starting phi value and reflecting range
C  Returns reflection coords on sphere XR, Phi start, phi range
C  and reciprocal Lorentz factor
C
C
      IF (.NOT.QUICK) THEN
         CALL RPF_DSTAR (X12(1), Y12(1), Z12(1), X12(2), Y12(2), Z12(2),       
     +                   ROTS, ROTE, DSTPL2, NWMAX, S0, SCAX, 
     +                    ISYN, DIVH, DIVV, ETA, DELAMB, DELCOR, 
     +                   IAX_H, IAX_V,
     +                   IR_FLAG, INR_FLAG, NIMG, IFIRST, XRLPE, 
     +                   PHI, PHIW, RLORF)

C
C Get detector coordinates & save spot as needed (via user supplied routine)
C
         CALL SAVSPOT (IHKL, IR_FLAG, INR_FLAG,  NIMG, IFIRST,
     +                 XRLPE, PHI, PHIW, RLORF, DST2MX, KERR)
         IF (KERR.NE.0) THEN
            IERR = KERR
            RETURN
         END IF
      ELSE
         CALL RPF_DSTAR_QUICK (X12(1), Y12(1), Z12(1),       
     +                         ROTS, ROTE, S0, SCAX_UN, 
     +                         IR_FLAG, INR_FLAG, XRLPE, PHI) 

C
C Get detector coordinates & save spot as needed (via user supplied routine)
C
         CALL SAVSPOT (IHKL, IR_FLAG, INR_FLAG, XRLPE, PHI, DST2MX,
     +                 KERR)
         IF (KERR.NE.0) THEN
            IERR = KERR
            RETURN
         END IF
      END IF 
C
C  End of IR loop
C
 440                 CONTINUE
C
C  End of IQ loop
C
 450              CONTINUE
C
C  End of IP loop
C
 460           CONTINUE
C
      RETURN
      END
