CD-Section: Unique Data Analysis
C A routine is available for generating a lists of unique 
C reflections for a given cell and symmetry. It requires the use of the 
C keyword symmetry module (KSM) routines to handle the symmetry and the set 
C of Fortran/C routines for handling unique reflection lists. The latter
C routines enable reflection counts of predicted or observed data to be
C accumulated for the purpose of gathering unique data coverage statistics.
CD-end

C LAST UPDATE: 
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C**************************
C**     RPF_UNQ_GEN      **
C**************************
C
CD-Routine: Generate unique reflections list - RPF_UNQ_GEN
C This routine will generate a list of the unique relections for a given
C resolution range and cell based on the current KSM symmetry data which must
C have been be defined. The unique reflections list is stored internally and
C may be accessed via routine which are part of the set of routines for
C handling unique reflections lists. 
CD-end
CD-Fortran:
      SUBROUTINE RPF_UNQ_GEN (MINDX_UNQ, CELL, WAVE, RESMIN, RESMAX,  
     +                        NUNIQUE, IERR)
CD-end
C
C Purpose: Generate list of unique reflections
C
C Author:  John W. Campbell, September 1995
C
C Arguments:
C
      INTEGER MINDX_UNQ
      REAL CELL(6)
      REAL WAVE
      REAL RESMIN
      REAL RESMAX
      INTEGER NUNIQUE
      INTEGER IERR
      
CD-Parameters:
C  MINDX_UNQ i (R/W)   Index for accessing unique data list; if -1 on input
C                      then a new list is generated. Otherwise it must be
C                      set equal to that returned from a previous call
C                      to RPF_UNQ_GEN. Returns the index found (-1 if error)
C  CELL(6)   r (R)     The real cell parameters in Angstroms and degrees
C  WAVE      r (R)     The wavelength
C  RESMIN    r (R)     Minimum resolution limit in Angstroms  
C  RESMAX    r (R)     Maximum resolution limit in Angstroms 
C  NUNIQUE   i (W)     No. of unique reflections generated
C  IERR      i (W)     Error flag =0  OK, 
C                                 =1  No symmetry defined,
C                                 =2  too may reflections to store
C                                 =-1 Cannot allocate required memory
C        
C                      Resolution limits may be in either order; one may be
C                      0.0 to indicate a low resolution limit of infinity
CD-end
C
C====== Parameter statements
C 

C
C====== GLOBALS:
C

C
C====== FUNCTION TYPES:
C
      INTEGER INASU
      REAL STHLSQ
C
C====== LOCALS:
C
      LOGICAL ABSENT
      INTEGER IH
      INTEGER IK
      INTEGER IL
      INTEGER IHKL(3)
      INTEGER MLAUE
      INTEGER IHMIN
      INTEGER IHMAX
      INTEGER IKMIN
      INTEGER IKMAX
      INTEGER ILMIN
      INTEGER ILMAX
      INTEGER NTWO10
      INTEGER NTWO20
      INTEGER IFLAG
      INTEGER NSYM
      INTEGER NUM_UNQ
      REAL RSMIN
      REAL RSMAX
      REAL SMIN
      REAL SMAX
      REAL TEMP
      REAL STHLMIN
      REAL STHLMAX
      REAL STHL
      REAL BMAT(3,3)
      REAL XRL
      REAL YRL
      REAL ZRL

C     LOGICAL ABSENT          !Systematic absence flag
C     INTEGER IH              !'h' index loop counter
C     INTEGER IK              !'k' index loop counter
C     INTEGER IL              !'l' index loop counter
C     INTEGER IHKL(3)         !Reflection indices
C     INTEGER MLAUE           !Laue group number
C     INTEGER IHMIN           !Minimum 'h' limit
C     INTEGER IHMAX           !Maximum 'h' limit
C     INTEGER IKMIN           !Minimum 'k' limit
C     INTEGER IKMAX           !Maximum 'k' limit
C     INTEGER ILMIN           !Minimum 'l' limit
C     INTEGER ILMAX           !Maximum 'l' limit
C     INTEGER NTWO10          !2**10
C     INTEGER NTWO20          !2**20
C     INTEGER IFLAG           !Status flag
C     INTEGER NSYM            !No. of symmetry operators
C     INTEGER NUM_UNQ         !Counter for no. of unique reflns
C     REAL RSMIN              !Minimum (low) resolution limit in Angstroms
C     REAL RSMAX              !Maximum (low) resolution limit in Angstroms
C     REAL SMIN               !Minimum 4sin**2theta/lambda**2 value
C     REAL SMAX               !Maximum 4sin**2theta/lambda**2 value
C     REAL TEMP               !Temporary variable
C     REAL STHLMIN            !Minimum sin**2theta/lambda**2 value
C     REAL STHLMAX            !Maximum sin**2theta/lambda**2 value
C     REAL STHL               !sin**2theta/lambda**2 value
C     REAL BMAT(3,3)          !'B' matrix
C     REAL XRL                !Reciprocal lattice 'x' coordinate
C     REAL YRL                !Reciprocal lattice 'y' coordinate
C     REAL ZRL                !Reciprocal lattice 'z' coordinate

      DATA NTWO20/1048576/, NTWO10/1024/
C
C-------------------------------------------------------------------------------
C
C
C====== Initialisations
C
      IERR = 0
      NUNIQUE = 0
      CALL KSM_GET_NSYM (NSYM, IFLAG)
      IF (NSYM.EQ.0) THEN
         IERR = 1
         RETURN
      END IF
      NUM_UNQ = 0
      CALL KSM_ASUSET (MLAUE)
      RSMIN = RESMIN
      RSMAX = RESMAX
      IF (RSMIN.LE.RSMAX) THEN
         TEMP = RSMAX
         RSMAX = RSMIN
         RSMIN = TEMP
      END IF
      IF (RSMAX.NE.0.0) THEN
         SMIN = 1.0/RSMIN**2
         SMAX = 1.0/RSMAX**2
      ELSE
         RSMAX = RSMIN
         RSMIN = 0.0
         SMIN = 0.0
         SMAX = 1.0/RSMAX**2
      END IF
      STHLMIN = SMIN/4.0
      STHLMAX = SMAX/4.0
C
C====== Set up 'B' matrix
C
      CALL RPF_SET_BMATRIX (CELL, WAVE, BMAT)
C
C====== Create or clear in-memory records list
C
      IF (MINDX_UNQ.LT.0) THEN
         CALL UNQF_REFLIST_INIT (1000, 1000, MINDX_UNQ)
         IF (MINDX_UNQ.LT.0) THEN
            IERR = -1
            MINDX_UNQ = -1
            RETURN
         END IF
      ELSE
         CALL UNQF_REFLIST_CLEAR (MINDX_UNQ, IERR)
      END IF
C
C====== Get Laue group and h, k, l limits
C
      CALL RPF_UNQ_LIMS (MLAUE, CELL, RSMAX, IHMIN, IHMAX, 
     +                   IKMIN, IKMAX, ILMIN, ILMAX, IERR)
C
C====== Prepare for resolution calculations
C
      CALL SETRSL (CELL(1),CELL(2),CELL(3),CELL(4),CELL(5),CELL(6))
C
C====== Generate reflection list
C
      DO 200 IH = IHMIN, IHMAX
         DO 150 IK = IKMIN, IKMAX
            DO 100 IL = ILMIN, ILMAX
               IF (IH.EQ.0.AND.IK.EQ.0.AND.IL.EQ.0) GO TO 100
               IHKL(1) = IH
               IHKL(2) = IK
               IHKL(3) = IL
               IF (INASU(IHKL, MLAUE).LE.0) GO TO 100
               CALL KSM_SYSABS (IH, IK, IL, ABSENT)
               IF (ABSENT) GO TO 100
               STHL = STHLSQ (IH, IK, IL)
               IF (STHL.LT.STHLMIN.OR.STHL.GT.STHLMAX) GO TO 100
               XRL = IH*BMAT(1,1) + IK*BMAT(1,2) + IL*BMAT(1,3)
               YRL = IH*BMAT(2,1) + IK*BMAT(2,2) + IL*BMAT(2,3)
               ZRL = IH*BMAT(3,1) + IK*BMAT(3,2) + IL*BMAT(3,3)
               CALL UNQF_REFLIST_ADDREF (MINDX_UNQ, IH, IK, IL, XRL,
     +                                   YRL, ZRL, IERR)
               IF (IERR.LT.0) RETURN
               NUM_UNQ = NUM_UNQ + 1
100         CONTINUE
150      CONTINUE
200   CONTINUE
C
C====== Sort data on h,k,l
C
400   NUNIQUE = NUM_UNQ
      IF (NUM_UNQ.LE.1) RETURN
      CALL UNQF_REFLIST_SORTHKL (MINDX_UNQ, IERR)
      RETURN
      END
